<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
    <xsl:output method="html"/>

    <!-- Variable passed from ant with the creation time -->
  <xsl:param name="reporttime"/>
  <xsl:param name="resources.dir" select="'./webtest'"/>

  <xsl:template match="/">

        <!-- HTML prefix -->
        <HTML>
        <HEAD>
    <meta http-equiv="content-style-type" content="text/css"/>
    <LINK rel="stylesheet" type="text/css" href="{$resources.dir}/css/report.css"></LINK>
        <TITLE>Canoo WebTest - Test Result Overview</TITLE>
        </HEAD>
  <BODY>
    <!-- outer holder table -->
    <table border="0" width="96%" cellspacing="0" style="margin-left:10px;">
  <tr>
    <td width="10"> </td>
    <td>
      <table border="0">
    <!-- CS Header -->
    <tr >
            <td class="Topmenu1Container" >
              <br/>
                <H2>Canoo WebTest Report</H2>
            </td>

      <td class="Topmenu1Container" align="right" valign="bottom">
            Report created at <xsl:value-of select="$reporttime"/>
      </td>
    </tr>

    <!-- spacer -->
    <tr><td>&#160; </td></tr>

        <!-- Header and summary table -->
        <tr align="left">
          <xsl:call-template name="StepStatistics"/>
        </tr>
        <tr>
          <xsl:call-template name="OverviewTable"/>
    </tr>

        <!-- All individual test results -->
        <xsl:apply-templates />

        <!-- Footer & fun -->
              <tr>
                <td class="footer" colspan="2">
            <hr/>
            Created using
            <a href="http://webtest.canoo.com">CanooWebTest</a>
          </td>
        </tr>
      </table>
    </td>
  </tr>
        </table>

        <!-- HTML postfix -->
        </BODY>
        </HTML>

    </xsl:template>

    <xsl:template name="OverviewTable">

        <xsl:variable name="testresult.list" select="summary/testresult"/>
        <xsl:variable name="duration.total"  select="sum(//testresult/results/step/result/completed/@duration)"/>

        <td valign="top">
        <a>
            <xsl:attribute name="name">overview</xsl:attribute>
        </a>
            <h4>Test Scenario Overview (<xsl:call-template name="time">
                  <xsl:with-param name="msecs" select="$duration.total"/>
              </xsl:call-template>)
          </h4>

        <!--
            Create summary table entries choosing the td-class depending on sucsessful yes/no
            and create a link to the appropriate detail section (e.g. #testspec1).
        -->
        <table cellpadding="5" border="0" cellspacing="0">
            <tr><th>No</th><th>Result</th><th>Name</th><th>Duration</th><th>%</th><th>&#160;</th><th>Graph</th></tr>
            <xsl:for-each select="$testresult.list">
              <xsl:variable name="actual.testspec.name" select="@testspecname"/>
                <tr>
                    <td class="light" align="right">
                        <xsl:number /></td>
                    <td class="light" align="center">
                        <xsl:if test="@successful='no'"><img src="{$resources.dir}/images/todo.gif"/></xsl:if>
                        <xsl:if test="@successful='yes'"><img src="{$resources.dir}/images/ok.gif"/></xsl:if>
                    </td>
                    <td class="light" nowrap="true">
                        <a>
                            <xsl:attribute name="href">#testspec<xsl:number /></xsl:attribute>
                            <xsl:value-of select="@testspecname"/>
                        </a>
                    </td>
                    <xsl:variable name="actual.testspec.name" select="@testspecname"/>
                    <xsl:variable name="actual.testspec.duration"
                      select="sum(self::node()[@testspecname=$actual.testspec.name]/results/step/result/completed/@duration)"/>
                    <td class="light" align="right">
                      <xsl:call-template name="time">
                    <xsl:with-param name="msecs" select="$actual.testspec.duration"/>
                </xsl:call-template></td>
                    <td class="light" align="right">
                      <xsl:value-of select="round($actual.testspec.duration * 100 div $duration.total)"/></td>

                    <xsl:call-template name="colorBar">
          <xsl:with-param name="percentage" select="$actual.testspec.duration * 100 div $duration.total"/>
          <xsl:with-param name="color" select="'lightblue'"/>
        </xsl:call-template>

                </tr>
            </xsl:for-each>
        </table>
    </td>
    </xsl:template>

    <xsl:template name="StepStatistics">
        <td valign="top" >
        <table cellpadding="0" border="0" cellspacing="2" width="100%"><tr><td valign="top" width="50%">

            <H4>Result Summary</H4>

    <xsl:variable name="steps.total" select="count(//results/step)"/>
    <xsl:variable name="steps.ok" select="count(//results/step/result/completed)"/>
    <xsl:variable name="steps.failed" select="count(//results/step/result/failed)"/>
    <xsl:variable name="steps.else" select="count(//results/step/result/notexecuted)"/>

        <table cellpadding="3" border="0" cellspacing="0">
            <tr><th align="right">Result</th><th align="right">#</th><th align="right">%</th><th>&#160;</th><th>Graph</th></tr>
                <tr>
                    <td class="light" ><img src="{$resources.dir}/images/ok.gif" alt="Steps which executed successfully"/></td>
                    <td class="light" align="right"><xsl:value-of select="$steps.ok"/></td>
                    <td class="light" align="right"><xsl:value-of select="round($steps.ok * 100 div $steps.total)"/></td>
      <xsl:call-template name="colorBar">
      <xsl:with-param name="percentage" select="$steps.ok * 100 div $steps.total"/>
      <xsl:with-param name="color" select="'lightgreen'"/>
      </xsl:call-template>

                </tr>
        <tr>
                    <td class="light" ><img src="{$resources.dir}/images/todo.gif" alt="Steps which failed"/></td>
                    <td class="light" align="right"><xsl:value-of select="$steps.failed"/></td>
                    <td class="light" align="right"><xsl:value-of select="round($steps.failed * 100 div $steps.total)"/></td>
      <xsl:call-template name="colorBar">
      <xsl:with-param name="percentage" select="$steps.failed * 100 div $steps.total"/>
      <xsl:with-param name="color" select="'red'"/>
      </xsl:call-template>
        </tr><tr>
                    <td class="light" ><img src="{$resources.dir}/images/optional.gif" alt="Steps which were not executed"/></td>
                    <td class="light" align="right"><xsl:value-of select="$steps.else"/></td>
                    <td class="light" align="right"><xsl:value-of select="round($steps.else * 100 div $steps.total)"/></td>
      <xsl:call-template name="colorBar">
      <xsl:with-param name="percentage" select="$steps.else * 100 div $steps.total"/>
      <xsl:with-param name="color" select="'yellow'"/>
      </xsl:call-template>
</tr>
         <tr>
                    <td class="light" ><b>Sum</b></td>
                    <td class="light" align="right"><b>&#160;<xsl:value-of select="$steps.total"/></b></td>
                    <td class="light" align="right"><b>&#160;100</b></td>
                    <td class="light" >&#160;</td>
                    <td class="light" >&#160;</td>
        </tr>

        </table>

        </td><td valign="top">

        <H4>Server Roundtrip Timing Profile</H4>
<!--        ================================ server roundtrip statistic table =============================    -->

        <xsl:variable name="invoke.steps" select="//results/step[parameter[@name='stepType'][@value='invoke']]"/>
        <xsl:variable name="clicklink.steps" select="//results/step[parameter[@name='stepType'][@value='clicklink']]"/>
        <xsl:variable name="clickbutton.steps" select="//results/step[parameter[@name='stepType'][@value='clickbutton']]"/>
        <xsl:variable name="duration.steps" select="$invoke.steps | $clicklink.steps | $clickbutton.steps"/>

        <xsl:variable name="last.steps" select="count($duration.steps/result/completed[@duration > 30000])"/>
        <xsl:variable name="third.steps" select="count($duration.steps/result/completed[@duration > 10000][30000 >= @duration])"/>
        <xsl:variable name="second.steps" select="count($duration.steps/result/completed[@duration > 5000][10000 >= @duration])"/>
        <xsl:variable name="first.steps" select="count($duration.steps/result/completed[@duration > 1000][5000 >= @duration])"/>
        <xsl:variable name="begin.steps" select="count($duration.steps/result/completed[1000 >= @duration])"/>

        <table cellpadding="3" border="0" cellspacing="0">
            <tr><th align="center">Secs</th><th align="right">#</th><th align="right">%</th><th>&#160;</th><th>Histogram</th></tr>

            <xsl:call-template name="histogramRow">
                <xsl:with-param name="label"> &#160;0 - &#160;1 </xsl:with-param>
                <xsl:with-param name="steps" select="$begin.steps" />
                <xsl:with-param name="duration.steps" select="$duration.steps"/>
            </xsl:call-template>

            <xsl:call-template name="histogramRow">
                <xsl:with-param name="label"> &#160;1 - &#160;5 </xsl:with-param>
                <xsl:with-param name="steps" select="$first.steps" />
                <xsl:with-param name="duration.steps" select="$duration.steps"/>
            </xsl:call-template>

            <xsl:call-template name="histogramRow">
                <xsl:with-param name="label"> &#160;5 - 10 </xsl:with-param>
                <xsl:with-param name="steps" select="$second.steps" />
                <xsl:with-param name="duration.steps" select="$duration.steps"/>
            </xsl:call-template>

            <xsl:call-template name="histogramRow">
                <xsl:with-param name="label"> 10 - 30 </xsl:with-param>
                <xsl:with-param name="steps" select="$third.steps" />
                <xsl:with-param name="duration.steps" select="$duration.steps"/>
            </xsl:call-template>

            <xsl:call-template name="histogramRow">
                <xsl:with-param name="label"> &#160;&#160; > 30 </xsl:with-param>
                <xsl:with-param name="steps" select="$last.steps" />
                <xsl:with-param name="duration.steps" select="$duration.steps"/>
            </xsl:call-template>

         <tr>
                    <td class="light" ><b>Sum</b></td>
                    <td class="light" align="right"><b>&#160;<xsl:value-of select="count($duration.steps)"/></b></td>
                    <td class="light" align="right"><b>&#160;100</b></td>
                    <td class="light" >&#160;</td>
                    <td class="light" >&#160;</td>
        </tr>
         <tr>
                    <td class="light" ><b>Avg</b></td>
                    <td class="light" align="right"><b>&#160;</b></td>
                    <td class="light" align="right"><b>&#160;</b></td>
                    <td class="light" >&#160;</td>
                    <td class="light" ><b>&#160;<xsl:value-of
                        select="round(sum($duration.steps/result/completed/@duration) div 1000 div count($duration.steps))"/></b></td>
        </tr>

        </table>

  </td></tr></table>

    </td>
    </xsl:template>

  <xsl:template name="histogramRow">
      <xsl:param name="label"/>
      <xsl:param name="steps"/>
      <xsl:param name="duration.steps"/>
        <tr>
        <td class="light" nowrap="true"> <xsl:value-of select="$label"/> </td>
        <td class="light" align="right"><xsl:value-of select="$steps"/></td>
        <td class="light" align="right"><xsl:value-of select="round($steps * 100 div count($duration.steps))"/></td>
        <xsl:call-template name="colorBar">
            <xsl:with-param name="percentage" select="$steps * 100 div count($duration.steps)"/>
            <xsl:with-param name="color" select="'lightblue'"/>
        </xsl:call-template>
        </tr>
  </xsl:template>

  <xsl:template name="colorBar">
      <xsl:param name="percentage"/>
      <xsl:param name="color"/>

    <td width="2px" class="light">&#160;</td>
    <td width="80%" class="light">
      <xsl:if test="$percentage > 0">
      <table border="0" cellpadding="0" cellspacing="0">
        <xsl:attribute name="width">
          <xsl:value-of select="concat($percentage, '%')"/>
        </xsl:attribute>
        <tr>
          <td bgcolor="{$color}">&#160;</td>
        </tr>
      </table>
      </xsl:if>
    </td>
  </xsl:template>

  <xsl:template name="time">
      <xsl:param name="msecs"/>
        <xsl:variable name="base" select="round($msecs div 1000)"/>
        <xsl:variable name="hours" select="floor($base div 3600)"/>
        <xsl:variable name="mins" select="floor(($base - $hours*3600) div 60)"/>
        <xsl:variable name="secs" select="floor(($base - $hours*3600) - $mins*60)"/>
        <xsl:if
         test="10 > $hours">0</xsl:if><xsl:value-of select="$hours"/>:<xsl:if
         test="10 > $mins">0</xsl:if><xsl:value-of select="$mins"/>:<xsl:if
         test="10 > $secs">0</xsl:if><xsl:value-of select="$secs"/>
  </xsl:template>


    <xsl:template match="testresult">

        <xsl:variable name="testspec.name" select="@testspecname"/>

    <tr><td colspan="2">
            <hr/>
        </td></tr>

    <tr><td>

    <!-- general info left -->
        <!-- Header and red/green box for sucess/failure overview-->
        <blockquote>
        <a>
            <xsl:attribute name="name">testspec<xsl:number /></xsl:attribute>
        </a>
          <H4>
                      <!-- the success indicator -->
            <xsl:if test="@successful='no'">
               <img src="{$resources.dir}/images/todo.gif"/>
            </xsl:if>

            <xsl:if test="@successful='yes'">
               <img src="{$resources.dir}/images/ok.gif"/>
            </xsl:if>

             &#160; <xsl:value-of select="$testspec.name"/> </H4>

        Test started at <xsl:value-of select="@starttime"/><br/>
        Based on <xsl:apply-templates select="config"/>
        </blockquote>

    </td>
  </tr>

  <tr>
    <td>
        <!-- The test step results in sequence below -->

          <xsl:apply-templates select="results"/>
    </td>
  </tr>
  <tr>
    <td colspan="2">
      <xsl:apply-templates select="results/failure"/>
      <xsl:apply-templates select="results/error"/>
    </td>
  </tr>

        <!-- Create link back to overview (top) -->
        <tr><td><a href="#overview">Back to Test Report Overview</a></td></tr>

    </xsl:template>

    <!-- Individual templates -->
    <xsl:template match="config">
      <xsl:variable name="configHref"><xsl:value-of
        select="parameter[@name='protocol']/@value"/>://<xsl:value-of
        select="parameter[@name='host']/@value"/>:<xsl:value-of
        select="parameter[@name='port']/@value"/>/<xsl:value-of
        select="parameter[@name='basepath']/@value"/>/</xsl:variable>
      <a><xsl:attribute name="href"><xsl:value-of select="$configHref"/></xsl:attribute>
         <xsl:attribute name="target">_new</xsl:attribute>
        <xsl:value-of select="$configHref"/></a>
    </xsl:template>

    <xsl:template match="parameter[@name='resultfilename']">
        <tr>
            <td colspan="2">
                <a>
                    <xsl:attribute name="href">
                        <xsl:text>webtest/response/</xsl:text><xsl:value-of select="@value"/>
                    </xsl:attribute>
                    <xsl:attribute name="target">_new</xsl:attribute>
                    Resulting page
                </a>
            </td>
        </tr>
    </xsl:template>

    <xsl:template match="parameter">
        <tr>
            <!-- do not show regegex='false' -->
            <xsl:if test="@name='regex'">
                <xsl:if test="@value='false'">
                    <td class="light">&#160;</td>
                    <td class="light">&#160;</td>
                </xsl:if>
                <xsl:if test="@value!='false'">
                    <td class="light"><b><xsl:value-of select="@name"/></b></td>
                    <td class="light"><xsl:value-of select="@value"/></td>
                </xsl:if>
            </xsl:if>
            <xsl:if test="@name!='regex'">
                <!-- zeige nicht 'null' -->
                <td class="light"><b><xsl:value-of select="@name"/></b></td>
                <td class="light">
                    <xsl:if test="@name='text'">
                        <xsl:if test="@value='null'">&#160;</xsl:if>
                        <xsl:if test="@value!='null'"><xsl:value-of select="@value"/></xsl:if>
                    </xsl:if>
                    <xsl:if test="@name!='text'">
                        <xsl:value-of select="@value"/>
                    </xsl:if>
                </td>
            </xsl:if>
        </tr>
    </xsl:template>

    <xsl:template match="results">
        <xsl:if test="count(step) > 0">
            <table cellpadding="3" border="0" cellspacing="0" >
                <tr><th>No</th><th>Result</th><th>Name</th><th>Parameter</th><th>Duration</th></tr>
                <xsl:apply-templates select="step"/>
            </table>
        </xsl:if>
    </xsl:template>


    <xsl:template match="step">
<tr>
<!-- ###################################################################### -->
<!-- tj: create image to show/hide substeps if row contains substeps and add onclick and groupStep attribute to image -->
  <!-- td class="light" style="border-bottom:1px inset black;">
    <table cellspacing="0" cellpadding="0" align="right">
      <tr>
        <td align="right" class="light"><b>&#160;<xsl:number/></b></td>
      </tr>
  <xsl:if test="parameter[@name='stepType'][@value='group']">
          <tr>
            <td align="right">
              <img>
                <xsl:attribute name="onclick">changeLevelOfDetailForGroup(this)</xsl:attribute>
                <xsl:attribute name="groupStep">true</xsl:attribute>
                <xsl:attribute name="src">./images/more.gif</xsl:attribute>
              </img>
            </td>
          </tr>
      </xsl:if>
    </table>
  </td -->
<!-- ###################################################################### -->



        <xsl:apply-templates select="result"/>
        <td  style="border-bottom:1px inset black;" valign="top" class="light">
            <table cellpadding="0" cellspacing="0">

  <tr>
  <td nowrap="true"><b><xsl:value-of select="parameter[@name='stepType']/@value"/></b></td></tr>
  <tr>
  <td>
                    <!-- zeige nicht title unknown-->
                    <xsl:if test="parameter[@name='stepId']/@value='&lt;unknown&gt;'">
                        &#160;
                    </xsl:if>
                    <xsl:if test="parameter[@name='stepId']/@value!='&lt;unknown&gt;'">
                        <xsl:value-of select="parameter[@name='stepId']/@value"/>
                    </xsl:if>
                </td></tr>
              <xsl:apply-templates select="parameter[@name='resultfilename']"/>
            </table>
        </td>
        <td  style="border-bottom:1px inset black;" valign="top" class="light">
          <table cellpadding="2" cellspacing="0">
                <xsl:apply-templates select="parameter[@name!='stepType' and @name!='stepId' and @name!='resultfilename']"/>
          </table>
          <!-- Write new table for contained steps (if any) -->
          <xsl:if test="count(step) > 0">
      <xsl:call-template name="renderStepTable" />
    </xsl:if>
  </td>
        <td style="border-bottom:1px inset black;" valign="top" align="right" class="light"> <xsl:value-of select="result/completed/@duration"/></td>
       </tr>
    </xsl:template>

    <xsl:template name="renderStepTable">
      <table cellpadding="3" cellspacing="0">
        <tr><th>No</th><th>Result</th><th>Name</th><th>Parameter</th><th>Duration</th></tr>
          <xsl:apply-templates select="step"/>
        </table>
    </xsl:template>


    <xsl:template match="result">
        <xsl:if test="completed">
    <td  style="border-bottom:1px inset black;" class="light" align="center">

<!-- ###################################################################### -->
<!-- tj: set status of a step of type 'group' to 'failed' if one of its nested steps fails  -->
<!-- tj: set status to 'ok' if step is not of type 'group' or if all substeps of a step of type 'group' have been executed successfully. -->
        <xsl:if test="preceding-sibling::*[@name='stepType'][@value='group']/following-sibling::step/result/failed">
                    <img src="{$resources.dir}/images/todo.gif"/>
        </xsl:if>
        <xsl:if test="not(preceding-sibling::*[@name='stepType'][@value='group']/following-sibling::step/result/failed)">
                    <img src="{$resources.dir}/images/ok.gif"/>
        </xsl:if>
<!-- ###################################################################### -->

          </td>
        </xsl:if>
        <xsl:if test="failed">
                <td  style="border-bottom:1px inset black;" class="light" align="center">
                <img src="{$resources.dir}/images/todo.gif"/><br/>
          <a>
                    <xsl:attribute name="href">#error<xsl:number count="testresult"/></xsl:attribute>
                    <font size="-10">
                    Error
                    </font>
                </a>
          </td>
        </xsl:if>
        <xsl:if test="notexecuted">
            <td  style="border-bottom:1px inset black;" class="light" align="center">
                <img src="{$resources.dir}/images/optional.gif"/>
          </td>
        </xsl:if>
    </xsl:template>


    <xsl:template match="failure">

        <a>
            <xsl:attribute name="name">error<xsl:number count="testresult"/></xsl:attribute>
            <h2>Failure</h2>
        </a>
        <h3>Message</h3>

        <xsl:value-of select="@message"/>
        <br/><br/>

    </xsl:template>

    <xsl:template match="error">

        <a>
            <xsl:attribute name="name">error<xsl:number count="testresult"/></xsl:attribute>
            <h2>Error</h2>
        </a>
        <h3>Exception</h3>

        <xsl:value-of select="@exception"/>
          <h3>Exception</h3>

        <xsl:value-of select="@message"/>
          <h3>Stacktrace</h3>
          <PRE>
                <xsl:value-of select="text()" disable-output-escaping="no"/>
          </PRE>

    </xsl:template>


</xsl:stylesheet>
